
#include <stdio.h>
#include <cstdlib>
#include <sys/wait.h>
#include <qmessagebox.h>
#include <qdatetime.h>

#define AT_COMMAND "qat"
#define AUTO_MAIL "/home/QtPalmtop/bin/automail"
#define CONFIG_FILE "/Settings/mailconfig.conf"
#define TRIGGER "/var/spool/at/trigger"
#include "mailconfig.h"
//-------------------------------------------------------------------
MailConfig::MailConfig(QPEApplication *app)
	: QMainWindow( 0, "MailConfig")
{
	thisApp = app;

	widget = new MainWidget( this, "MainWidget" );
	setCentralWidget( widget );
	setCaption(tr("MailConfig"));

	widget->command->setText( tr( AUTO_MAIL ) );
	connect( widget->apply, SIGNAL(clicked()), SLOT(onApply()) );

	loadConfig();
}
//-------------------------------------------------------------------
void MailConfig::run()
{
	thisApp->setMainWidget( this );
	showMaximized();
}
//-------------------------------------------------------------------
void MailConfig::onApply()
{
	QString cmd;
	QString append = "";
	if( widget->disable->isChecked() )
	{
		cmd.sprintf("%s -d \"%s\"", AT_COMMAND, widget->command->text().ascii() );
		execute(cmd);
	}else
	{
		if( !QDate::isValid( widget->year->value(), widget->month->value(), widget->day->value() ) )
		{
			QMessageBox::critical(0, "Fail", tr("դ̵Ǥ"));
			widget->year->setFocus();
			return;
		}

		if( !QTime::isValid( widget->hour->value(), widget->minute->value(), 0 ) )
		{
			QMessageBox::critical(0, "Fail", tr("̵֤Ǥ"));
			widget->hour->setFocus();
			return;
		}

		if( widget->enaHour1->value() == 0 &&
		    widget->enaMinute1->value() == 0 &&
		    widget->enaHour2->value() == 0 &&
		    widget->enaMinute2->value() == 0 )
		{
			//not use
		}else
		{
			if( !QTime::isValid( widget->enaHour1->value(), widget->enaMinute1->value(), 0 ) )
			{
				QMessageBox::critical(0, "Fail", tr("̵֤Ǥ"));
				widget->enaHour1->setFocus();
				return;
			}
			if( !QTime::isValid( widget->enaHour2->value(), widget->enaMinute2->value(), 0 ) )
			{
				QMessageBox::critical(0, "Fail", tr("̵֤Ǥ"));
				widget->enaHour2->setFocus();
				return;
			}
			append.sprintf(" %02d:%02d-%02d:%02d",
				widget->enaHour1->value(), widget->enaMinute1->value(),
				widget->enaHour2->value(), widget->enaMinute2->value());
		}
		cmd.sprintf("%s %04d/%02d/%02d-%02d:%02d %d \"%s\"",
			AT_COMMAND,
			widget->year->value(), widget->month->value(), widget->day->value(),
			widget->hour->value(), widget->minute->value(),
			widget->interval->value(),
			widget->command->text().ascii()
		);
		cmd += append;
		execute(cmd);
	}
}
//-------------------------------------------------------------------
void MailConfig::execute(QString command)
{
	if( QMessageBox:: warning(this, "Command execute", tr("%1\nޥɤ¹Ԥޤ\nǤ").arg(command), QMessageBox::Yes, QMessageBox::No) == QMessageBox::No )
	{
		return;
	}
	saveConfig();
	int status = std::system(command.utf8());
	if (WEXITSTATUS(status) != 0)
	{
		QMessageBox::critical(0, "Fail", tr("ޥɼ¹Ԥ˼Ԥޤ"));
	}else
	{
		saveConfig();
		trigger();
		QMessageBox::information(this, "Apply", tr("ޥɤ¹Ԥޤ"), QMessageBox::Ok);
	}
}
//-------------------------------------------------------------------
void MailConfig::loadConfig()
{
	QString file = QString(getenv("HOME")) + CONFIG_FILE;
	Config cfg(file, Config::File);

	cfg.setGroup("Config");
	QString command = cfg.readEntry("command");
	if( !command.isEmpty() ){
		widget->command->setText( command );
	}

//	widget->year->setValue    ( loadConfigValue(cfg, widget->year,     "year") );
//	widget->month->setValue   ( loadConfigValue(cfg, widget->month,    "month") );
//	widget->day->setValue     ( loadConfigValue(cfg, widget->day,      "day") );
//	widget->hour->setValue    ( loadConfigValue(cfg, widget->hour,     "hour") );
//	widget->minute->setValue  ( loadConfigValue(cfg, widget->minute,   "minute") );
	widget->interval->setValue( loadConfigValue(cfg, widget->interval, "interval") );

	QString disable = cfg.readEntry("disable");
	if( disable == "1") widget->disable->setChecked( true ); else widget->disable->setChecked( false );

	cfg.setGroup("EnableTime");
	widget->enaHour1->setValue  ( loadConfigValue(cfg, widget->enaHour1,   "hour1") );
	widget->enaMinute1->setValue( loadConfigValue(cfg, widget->enaMinute1, "minute1") );
	widget->enaHour2->setValue  ( loadConfigValue(cfg, widget->enaHour2,   "hour2") );
	widget->enaMinute2->setValue( loadConfigValue(cfg, widget->enaMinute2, "minute2") );
}
//-------------------------------------------------------------------
int MailConfig::loadConfigValue(Config &cfg, QSpinBox *spn, QString key)
{
	bool ok;

	int value = cfg.readEntry(key).toInt(&ok);
	if( !ok ){
		value = spn->value();
	}
	return value;
}
//-------------------------------------------------------------------
void MailConfig::saveConfig()
{
	QString file = QString(getenv("HOME")) + CONFIG_FILE;
	Config cfg(file, Config::File);
	cfg.setGroup("Config");

	cfg.writeEntry("command", widget->command->text());
	cfg.writeEntry("year", widget->year->text());
	cfg.writeEntry("month", widget->month->text());
	cfg.writeEntry("day", widget->day->text());
	cfg.writeEntry("hour", widget->hour->text());
	cfg.writeEntry("minute", widget->minute->text());
	cfg.writeEntry("interval", widget->interval->text());
	cfg.writeEntry("disable", (widget->disable->isChecked())? "1":"0");

	cfg.setGroup("EnableTime");
	cfg.writeEntry("hour1", widget->enaHour1->text());
	cfg.writeEntry("minute1", widget->enaMinute1->text());
	cfg.writeEntry("hour2", widget->enaHour2->text());
	cfg.writeEntry("minute2", widget->enaMinute2->text());
}
//-------------------------------------------------------------------
void MailConfig::trigger()
{
	FILE *f;

	if( (f = fopen(TRIGGER, "w")) == NULL )
	{
		fprintf(stderr, "cannot write [%s].\n", TRIGGER);
		exit(1);
	}
	fprintf(f, "\n");
	fclose(f);
}
//-------------------------------------------------------------------
