#include <stdio.h>
#include <stdlib.h>
#include <qcopchannel_qws.h>
#include <qpe/qcopenvelope_qws.h>
#include <qpe/config.h>

#define WND_WIDTH 160
#define WND_HEIGHT 80
#define BTN_WIDTH 100
#define BTN_HEIGHT 60
#define CONFIG_FILE "/Settings/suspender.conf"
#include "suspender.h"
//-------------------------------------------------------------------
Suspender::Suspender(QPEApplication *app) : QMainWindow()
{
	thisApp = app;

	QString file = QString(getenv("HOME")) + CONFIG_FILE;
	Config cfg(file, Config::File);

	cfg.setGroup("Config");
	bool ok;
	timeout = cfg.readEntry("timeout").toInt(&ok);
	if( !ok || timeout < 3){
		timeout = 5;
	}
	counter = timeout;

	timer = new QTimer( this, "timer" );
	connect(timer, SIGNAL(timeout()), this, SLOT(timerEvent()));

	QCopChannel *channel = new QCopChannel("QPE/Network", this);
	connect(channel, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(networkMessage(const QCString&, const QByteArray&)));

	QCopChannel *resume = new QCopChannel("QPE/System", this);
	connect(resume, SIGNAL(received(const QCString&, const QByteArray&)),
		this, SLOT(systemMessage(const QCString&, const QByteArray&)));

	cancel = new QPushButton( this, "cancel" );
	cancel->setText( tr( "Cancel" ) );
	cancel->setGeometry((WND_WIDTH-BTN_WIDTH)/2, (WND_HEIGHT-BTN_HEIGHT)/2, BTN_WIDTH, BTN_HEIGHT);
	connect( cancel, SIGNAL(clicked()), this, SLOT(onCancel()) );
}
//-------------------------------------------------------------------
Suspender::~Suspender()
{
}
//-------------------------------------------------------------------
void Suspender::networkMessage(const QCString& msg, const QByteArray& /*data*/)
{
	QString s = QString(msg);
	if(s == "down()" && counter == timeout)
	{
		timer->start( 1000, false );
		thisApp->showMainDocumentWidget( this );
		setCaption( tr("suspend %1").arg(counter) );

		int w = thisApp->desktop()->width();
		int h = thisApp->desktop()->height();
		setGeometry((w-WND_WIDTH)/2, (h-WND_HEIGHT)/2, WND_WIDTH, WND_HEIGHT);
	}
}
//-------------------------------------------------------------------
void Suspender::systemMessage(const QCString& msg, const QByteArray& /*data*/)
{
	QString s = QString(msg);
	if(s == "resumed()")
	{
		thisApp->quit();
	}
}
//-------------------------------------------------------------------
void Suspender::timerEvent()
{
	counter--;
	setCaption( tr("suspend %1").arg(counter) );
	if(counter <= 0){
		system("su -c \"/bin/echo W >/var/spool/at/trigger\"");
		thisApp->quit();
		QCopEnvelope e(QCString("QPE/System"), QCString("execute(QString)"));
		e << QString("suspend");
	}
}
//-------------------------------------------------------------------
void Suspender::onCancel()
{
	thisApp->quit();
}
//-------------------------------------------------------------------
